
/* Picker.

   Display up to four items on the empeg display and wait
   for a selection via a button press or a timeout.
   Returns and outputs to stdio the number of that selection.

		picker [-t timeout default] \"First option\"
			   [\"Second option\"] [\"Third option\"] [\"Forth option\"]

		Returns the number of the button pressed (1-4) or 255 if error.

		If specified, if timeout seconds pass without a button press,
		option default is selected.

		Run with no parameters to not update the display.
		Run with -h switch for this message.


    Copyright (C) 2000 Rex K. Perkins <rexkp@pobox.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	History:

	 2Sep2000: Initial release.
	19Sep2000: Added support for remote control and 
			   'press any key to continue' mode.
	29Sep2000: Added timeout option.

*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <unistd.h>
#include <sys/mman.h>

#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/time.h>

#include "empeg_misc.h"
#include "ir.h"


#define BLIT(display_fd)        ioctl((display_fd), _IO('d', 0))
#define DISPLAY(display_fd, x)  ioctl((display_fd), _IOW('d', 1, int), (x))

/* Button map. Maps the key code of the left buttons,
 * shifted right by 1, to the return code.
 */
int button_map[4]={1,3,2,4};

int main(int argc, char *argv[])
{
	int       display_fd;
	int       ir_fd;
	caddr_t   display_mapped;
	int       line;
	int       col;
	int       char_count;
	int       index;
	int		  key_code;
	int		  byte_count;
	int		  exit_code = 0;
	int		  timeout = 0;
	int		  default_choice = 1;
	fd_set 	  probe;
	struct timeval tv;
	struct timeval * tv_ptr;
	int		  arg_index_of_first_option = 1;


	if ( (argc == 2) && (argv[1][0]=='-') && (argv[1][1]=='h') )
	{
		printf("picker [-t timeout default] \"First option\" [\"Second option\"] [\"Third option\"] [\"Forth option\"]\n");
		printf("\n");
		printf("Returns the number of the button pressed (1-4) or 255 if error.\n");
		printf("\n");
		printf("If specified, if timeout seconds pass without a button press, option default is selected.");
		printf("\n");
		printf("Run with no parameters to not update the display.\n");
		printf("Run with -h switch for this message.\n");
		return(255);
	}

	//Extract timeout, if any.
	if ( (argc > 3) && (argv[1][0]=='-') && (argv[1][1]=='t') )
	{
		timeout        = strtol(argv[2], NULL, 0);
		default_choice = strtol(argv[3], NULL, 0);
		if (default_choice < 1) default_choice = 1;
		if (default_choice > 4) default_choice = 4;
		arg_index_of_first_option += 3;	//Skip over switches
	}
	
	if (argc>1)
	{
		display_fd = open(EMPEG_DISPLAY_DEVICE, O_RDWR);
		if (display_fd == -1)
		{
			printf("Unable to open %s\n", EMPEG_DISPLAY_DEVICE);
			return(255);
		}

		display_mapped = mmap(0,
		                    EMPEG_DISPLAY_SIZE_BYTES,
		                    PROT_READ | PROT_WRITE,
		                    MAP_SHARED,
		                    display_fd,
		                    0);

		if (display_mapped == (caddr_t) -1)
		{
			printf("Unable to mmap %s\n", EMPEG_DISPLAY_DEVICE);
			return(255);
		}

		// Clear mapped display to 'black'.
		memset(display_mapped, 0, EMPEG_DISPLAY_SIZE_BYTES);

		for (line=0; ((line<FONT8X8_HEIGHT_CHARS) && ((line+arg_index_of_first_option)<argc) ); line++)
		{
			col = 0;
			index = 0;

			/* Display an icon in the first col of this line.
	 		 */
			putChar8x8(display_mapped,
			         col,
			         line,
			         true,
			         FONT8X8_FIRST_BUTTON_ICON+line);
			col++;

			while ((col<FONT8X8_WIDTH_CHARS) && 
				   (argv[line+arg_index_of_first_option][col-1] != 0) )
			{
				putChar8x8(display_mapped,
				         col,
				         line,
				         true,
				         argv[line+arg_index_of_first_option][col-1]);
				col++;
			}
		}


		BLIT(display_fd);
		DISPLAY(display_fd, 1);
	}

	/* Done with the display. Now read the keypad
     */

	ir_fd = open(EMPEG_IR_DEVICE, O_RDONLY&O_NDELAY&O_NONBLOCK);
	if (ir_fd == -1)
	{
		printf("Unable to open %s\n", EMPEG_IR_DEVICE);
		return(255);
	}

	//NOTE: After a select Linux leaves the amount of time not slept in the
	//timeout structure. We use that here to wait only for a total of
    //timeout. This is not portable to other unixes.
	if (timeout != 0)		
	{
		tv_ptr 		= &tv;
		tv.tv_sec	= timeout;
		tv.tv_usec 	= 0;
	}
	else
		tv_ptr = NULL;

	while (exit_code == 0)
	{
		//Do a select to see if a keypress is detected within the timeout period
		FD_SET(ir_fd, &probe);

		if (select(ir_fd+1, &probe, &probe,&probe, tv_ptr) <= 0)
		{
			//Timeout or error. Select default choice.
			exit_code = default_choice;
		}
		else
		{
			//There is a key code to read. Get it.
			byte_count = read(ir_fd, (char *) &key_code, sizeof(key_code));
			if ( ((key_code & 0xfff0) == 0) && !(key_code & IR_X_BUTTON_RELEASED))
			{
				/* Got a non-release code from a front panel button.
			     * Exit with appropriate return code.
		 		 */
				exit_code = button_map[(key_code >> 1) & 3];
			}
			else if ( ((key_code & 0xfff0) == (IR_0 & 0xfff0)) )
			{
				/* Got a remote control button press 1 to 4.
		 		 */
				exit_code = (key_code & 0xf);
			}

			/* Range check the key press for the number of options available.
			 * Allow any value if there is no text displayed.
	 		 */
			if ( (argc>1) && ((exit_code+arg_index_of_first_option) > argc) )
				exit_code = 0;
		}
	}

	close(ir_fd);

	if (argc>1)
	{
		/* Clear the screen on exit
	     */
		memset(display_mapped, 0, EMPEG_DISPLAY_SIZE_BYTES);
		BLIT(display_fd);
		munmap(display_mapped, EMPEG_DISPLAY_SIZE_BYTES);
		close(display_fd);
	}

	printf("%d",exit_code);

	return(exit_code);
}

